/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
 *  Author: Sebastian Scholz <digitalstrom@sunshine-studios.org>
 */

Ext.define('DSS.addon.Metering', {

	extend : 'DSS.addon.Framework',

	uses : ['DSS.addon.Metering.MainPanel', 'Ext.Component'],

	config : {
		appName : _('Metering'),
		appId : 'metering',
		appVersion : '0.2.2',
		appIcon : 'images/dss/default_icon.png',
		appLang : 'de_DE'
	},

	getHelp : function() {
		return Ext.create('Ext.Component', {
			loader : {
				url : 'locale/' + this.appLang + '/LC_MESSAGES/help.html',
				autoLoad : true,
				disableCaching : false
			}
		});
	},
	getContent : function() {
		return Ext.create("DSS.addon.Metering.MainPanel", this.config);
	}
});

Ext.onReady(function() {
	var metering = Ext.create('DSS.addon.Metering');
	metering.initPage();
});

/*######################################
 static configs
 ######################################*/

DSS.addon.Metering.time = {
		1 : _('secondly'),
		60 : _('minutely'),
		900 : _('15 minutely'),
		3600 : _('hourly'),
		10800 : _('5 hourly'),
		86400 : _('daily')
};

DSS.addon.Metering.steps = {
		1 : [Ext.Date.MINUTE, 1],
		60 : [Ext.Date.MINUTE, 30],
		900 : [Ext.Date.DAY, 1],
		3600 : [Ext.Date.DAY, 1],
		10800 : [Ext.Date.DAY, 1],
		86400 : [Ext.Date.DAY, 7]
};

DSS.addon.Metering.sceneNames = {
			65 : _("Panic"),
			67 : _("Standby"),
			68 : _("Deep off"),
			69 : _("Sleeping"),
			70 : _("Wake up"),
			71 : _("Coming home"),
			72 : _("Leave home"),
			73 : _("Door bell"),
			0 : _("Off"),
			5 : _("Scene 1"),
			17 : _("Scene 2"),
			18 : _("Scene 3"),
			19 : _("Scene 4"),
			6 : _("Area 1 on"),
			7 : _("Area 2 on"),
			8 : _("Area 3 on"),
			9 : _("Area 4 on"),
			32 : _("Off"),
			33 : _("Scene 11"),
			20 : _("Scene 12"),
			21 : _("Scene 13"),
			22 : _("Scene 14"),
			34 : _("Off"),
			35 : _("Scene 21"),
			23 : _("Scene 22"),
			24 : _("Scene 23"),
			25 : _("Scene 24"),
			36 : _("Off"),
			37 : _("Scene 31"),
			26 : _("Scene 32"),
			27 : _("Scene 33"),
			28 : _("Scene 34"),
			38 : _("Off"),
			39 : _("Scene 41"),
			29 : _("Scene 42"),
			30 : _("Scene 43"),
			31 : _("Scene 44")
		}

DSS.addon.Metering.refreshInterval = 2000;



/*######################################
 MainPanel
 ######################################*/
Ext.define('DSS.addon.Metering.MainPanel', {
	extend : 'Ext.panel.Panel',
	
	alias : 'widget.mainpanel',

	initComponent : function() {
		
		this.dSMView = Ext.create('DSS.addon.Metering.dSMView');
		this.centerPanel = Ext.create('DSS.addon.Metering.CenterPanel');
		
		Ext.apply(this, {
			//title : _('Metering Charts'),
			split : true,
			layout: {
				type: 'border'
			},
			items: [
            {
                items: this.dSMView,
                region: 'west',
                border: true,
                split: true,
                minWidth: 50,
                width: 200
            }, 
            this.centerPanel
            ]
		});

		DSS.addon.Metering.MainPanel.superclass.initComponent.apply(this, arguments);
	}
});


/*######################################
 dSMView
 ######################################*/
Ext.define('DSS.addon.Metering.dSMView', {
	extend : 'Ext.view.View',
	
	initComponent: function()
    {
        var me = this;
        this.circuitStore = Ext.create('DSS.addon.Metering.stores.CircuitViewStore');
        this.circuitStore.add({name: _("whole installation"), dsid: 0});
        this.circuitStore.load();		
              
		var circuitTemplate = new Ext.XTemplate('<tpl for=".">', '<div class="zone-wrap" id="zone-{dsid}">', '<span>', '{text}', '</span>', '</div>', '</tpl>', '<div class="x-clear"></div>');
                
		Ext.apply(this, {
			title : _("dSM Overview"),
			store : this.circuitStore,
			tpl : circuitTemplate,
			autoScroll : true,
			singleSelect : true,
			multiSelect : false,
			simpleSelect : false,
			trackOver : true,
			overItemClass : 'x-item-over',
			overClass : 'x-item-over',
			itemSelector : 'div.zone-wrap',
			emptyText : 'empty',
			listeners: {
                selectionchange: function(dv, nodes) {
                    this.getSelectedDSID();
            		this.findParentByType('mainpanel').centerPanel.repaint();
                }
            }

		});
		this.refresh();

        DSS.addon.Metering.dSMView.superclass.initComponent.apply(this, arguments);
    },

	getSelectedDSID: function()
    {
        if(this.getSelectionModel().getCount()===0){
        	this.getSelectionModel().select(0);
        }
                
        var node = this.getSelectionModel().getSelection();
        if (node !== null) {          
            var dsid = node[0].get('dsid');
            if(dsid === 0) {
            	//whole apartment is selected -> select next dsm
            	this.getSelectionModel().select(1);
            	node = this.getSelectionModel().getSelection();
            	if (node !== null) {
            		dsid = node[0].get('dsid');
            	}
            }
            return dsid;
        }
        else { 
            Ext.MessageBox.alert(_('Error'), _('please select a dSM!'));
       	}
       	
       	return 0;
    }
});


/*######################################
 CenterPanel
 ######################################*/

Ext.define('DSS.addon.Metering.CenterPanel', {
	extend : 'Ext.panel.Panel',

	initComponent : function() {
		var me = this;

		this.unit = '';

		this.resolutionStore = Ext.create('DSS.addon.Metering.stores.ResolutionStore');
		this.resolutionStore.load();

		if(this.resolutionStore.isLoading()) {
			this.resolutionStore.on('load', function() {
				me.initStores();
			});
		} else {
			this.initStores();
		}
		
		this.sceneNameStore = Ext.create('DSS.addon.Metering.stores.SceneNameStore');
		this.sceneNameStore.load();
		
		//this store is used to display the right events in the realtime charts
		this.apartmentStore = Ext.create('DSS.apartment.ApartmentStore');
		this.apartmentStore.load();
		
		this.typeStore = Ext.create('Ext.data.Store', {
			fields : ['type', 'name'],
			data : [{
				"type" : "consumption",
				"name" : _('Consumption')
			}, {
				"type" : "energy",
				"name" : _('Energy')
			}]
		});

		this.typeBox = Ext.create('Ext.form.field.ComboBox', {
			store : this.typeStore,
			labelAlign : 'top',
			editable : false,
			displayField : 'name',
			valueField : 'type',
			fieldLabel : _('unit')
		});
		this.typeBox.on('change', function() {
			me.repaint();
		});

		this.timeBox = Ext.create('Ext.form.field.ComboBox', {
			store : this.resolutionStore,
			labelAlign : 'top',
			editable : false,
			queryMode : 'local',
			displayField : 'name',
			valueField : 'resolution',
			fieldLabel : _('resolution')
		});
		this.timeBox.on('change', function() {
			me.repaint();
		});

		this.meteringLineStore = Ext.create('DSS.addon.Metering.MeteringStore');
		this.meteringColumnStore = Ext.create('DSS.addon.Metering.MeteringStore');
		
		this.meteringSeries = Ext.create('DSS.addon.Metering.LineSeries');
		this.meteringSeries.setCenterPanel(this);

		this.lineChart = Ext.create('Ext.chart.Chart', {
			store : this.meteringLineStore,
			autoScroll : true,
			axes : [{
				title : _('consumption') + ' (w)',
				type : 'Numeric',
				position : 'left',
				fields : ['value'],
				minimum : 0
			}, {
				type : 'Time',
				position : 'bottom',
				fields : 'date',
				title : _('Time'),
				dateFormat : 'd. M - G:i',
				groupBy : 'hour,minute,second',
				aggregateOp : 'sum'
			}],
			series : [ this.meteringSeries ],
			theme : 'MeteringTheme'
		});
		this.lineChart.on('refresh', function() {me.lineChart.series.get(0).updateMarkers();})
		this.originalLineZoom = null;
		
		this.columnChart = Ext.create('Ext.chart.Chart', {
			store : this.meteringColumnStore,
			axes : [{
				title : _('energy') + ' (Wh)',
				type : 'Numeric',
				position : 'left',
				fields : ['value'],
				minimum : 0
			}, {
				type : 'Time',
				position : 'bottom',
				fields : 'date',
				title : _('Time'),
				dateFormat : 'd. M - G:i',
				groupBy : 'hour,minute,second',
				aggregateOp : 'sum'
			}],
			series : [{
				type : 'column',
				xField : 'date',
				yField : 'value'
			}],
			theme : 'MeteringTheme'
		});
		
		this.exportCSVButton = Ext.create('Ext.button.Button', {
			text : _("export as CSV"),
			iconCls : 'icon-export',
			handler : function() {
				this.csvLabel = Ext.create('Ext.form.Label', {
					html : me.exportCSV()
				});

				var win = Ext.create('Ext.Window', {
					title : _("Copy the content of the messagebox below into a csv file"),
					layout : 'fit',
					hideBorders : true,
					height : 500,
					width : 700,
					minHeight : 250,
					minWidth : 500,
					autoScroll : true,
					scope : this,
					items : [this.csvLabel],
					buttons : [{
						text : _("Close"),
						scope : this,
						handler : function() {
							win.close();
							me.enable();
						}
					}]
				});

				win.on('beforedestroy', function() {
					me.enable();
				}, this);

				me.disable();
				win.show(this);
			}
		})

		
		this.thread = null;

		Ext.apply(this, {
			border : false,
			layout : 'fit',
			items : this.lineChart,
			tbar : [this.typeBox, this.timeBox, '->', this.exportCSVButton],
			region : 'center',
			border : true,
			split : true,
			minWidth : 600
		});

		DSS.addon.Metering.CenterPanel.superclass.initComponent.apply(this, arguments);
	},
	
	initStores : function() {
		for(i = 0; i < this.resolutionStore.getCount(); i++) {
			var record = this.resolutionStore.getAt(i);
			record.setName(DSS.addon.Metering.time[record.get('resolution')]);
		}
		this.typeBox.setValue('consumption');
		this.timeBox.bindStore(this.resolutionStore);
		this.timeBox.setValue(this.resolutionStore.getAt(1).get('resolution'));
		this.unit = 'W'
	},
	
	repaint : function() {
		var dsid = this.findParentByType('mainpanel').dSMView.getSelectedDSID();
		var url;
				
		if(this.typeBox.getValue() === 'consumption') {
			//check if something is selected, otherwise select second entry (first entry would enable realtime data)
			if(this.timeBox.getValue() === undefined){
				this.timeBox.setValue(this.resolutionStore.getAt(1).get('resolution'));
			}
			
			this.unit = 'W';
			this.lineChart.axes.get(1).step = DSS.addon.Metering.steps[this.timeBox.getValue()];
			url = '/json/metering/getValues?dsid=' + dsid + '&type=consumption&resolution=' + this.timeBox.getValue();
			
			if(this.items.get(0) != this.lineChart) {
				//show line chart, set column chart invisible due to performance reasons
				this.columnChart.setVisible(false);
				//this.columnChart.series.get(0).hideAll();
				this.lineChart.setVisible(true);
				//this.lineChart.series.get(0).showAll();
				this.remove(this.columnChart, false);
				this.add(this.lineChart);
			}
			
			//this.lineChart.restoreZoom();
						
			this.meteringLineStore.removeAll();
			this.meteringLineStore.getProxy().url = url;
			this.meteringLineStore.load();

			
		} else {	
			//check if something is selected, otherwise select first entry
			if(this.timeBox.getValue() === undefined){
				this.timeBox.setValue(this.resolutionStore.getAt(0).get('resolution'));
			}
			
			this.unit = 'Wh';
			this.columnChart.axes.get(1).step = DSS.addon.Metering.steps[this.timeBox.getValue()];
			url = '/json/metering/getValues?dsid=' + dsid + '&type=energy&resolution=' + this.timeBox.getValue();
			
			if(this.items.get(0) != this.columnChart) {
				// show column chart, set line chart invisible due to performance reasons
				this.lineChart.setVisible(false);
				//this.lineChart.series.get(0).hideAll();
				this.columnChart.setVisible(true);
				//this.columnChart.series.get(0).showAll();
				this.remove(this.lineChart, false);
				this.add(this.columnChart);
			}
			
			//this.columnChart.restoreZoom();
					
			this.meteringColumnStore.removeAll();
			this.meteringColumnStore.getProxy().url = url;
			this.meteringColumnStore.load();

		}
		
		
		if(this.typeBox.getValue() === 'consumption' && this.timeBox.getValue() === 1) {
			console.log('before start');
			this.stopThread();
			this.startThread();
			console.log('start thread');
		} else {
			this.stopThread();
		}
	},
	
	getUnit: function() {
		return this.unit;
	},
	
	exportCSV: function() {
		var csvString = '';
		csvString += '"' + _("Time") + '";"' + _("Value") + ' (' + _("in") + ' ' + this.unit + ')"<br>'
		if(this.typeBox.getValue() === 'consumption') {
			for(var i = 0; i < this.meteringLineStore.getCount(); i++) {
				var r = this.meteringLineStore.getAt(i);
				//csvString += '"' + new Date(r.get('date')).format('d.m.Y - G:i:s') + '";' + r.get('value') + '<br>';
				csvString += '"' + new Date(r.get('date')) + '";' + r.get('value') + '<br>';
			}
		} else {
			for(var i = 0; i < this.meteringColumnStore.getCount(); i++) {
				var r = this.meteringColumnStore.getAt(i);
				//csvString += '"' + new Date(r.get('date')).format('d.m.Y - G:i:s') + '";' + r.get('value') + '<br>';
				csvString += '"' + new Date(r.get('date')) + '";' + r.get('value') + '<br>';
			}
		}
		return csvString;
	},
	
	// starts the polling of the new metering data every x seconds via a setInterval method and an Ajax request
	// also jumps in the data series are detected and jump markers are added to the right position
	startThread : function() {
		var me = this;
		
		this.thread = setInterval(function() {
			console.log('run');
			Ext.Ajax.request({
				url : '/json/circuit/getConsumption?id=' + me.findParentByType('mainpanel').dSMView.getSelectedDSID(),
				// set the timeout to the refresh interval since we will do another request every 2 seconds
				timeout : DSS.addon.Metering.refreshInterval,
				success : function(response) {
					var jsonData = Ext.JSON.decode(response.responseText);
					if(jsonData.ok) {						
						var array = [];
						var record = [Math.round(+new Date() / 1000), jsonData.result.consumption]
						array.push(record);

						me.meteringLineStore.removeAt(0);
						me.meteringLineStore.loadData(array, true);

						/*   detect jumps
						* jump detection is based on the last 5 values
						* 
						*              c----d
						*             /
						*            /
						*           x 
						*          /
						*         /
						*   a----b
						* 
						* a jump is detected for c, if
						*      |a-b| < 10% of a
						*      |a-c| > 25% of a
						*      |c-d| < 10% of c
						*      |c-x| > 12.5% of c
						* 
						*/
						
						
						var count = me.meteringLineStore.getCount();
						var a = me.meteringLineStore.getAt(count - 5).get('value');
						var b = me.meteringLineStore.getAt(count - 4).get('value');
						var x = me.meteringLineStore.getAt(count - 3).get('value');
						var c = me.meteringLineStore.getAt(count - 2).get('value');
						var d = me.meteringLineStore.getAt(count - 1).get('value');
						
						if(Math.abs(a-b)<=0.1*a && Math.abs(c-d)<=0.1*c && Math.abs(a-c)>0.25*a && Math.abs(c-x)>0.125*c) {
							var diff = c - a;
							var diffString;
							if(diff>0){
								diffString = '+' + diff+' ' + me.unit;
							} else {
								diffString = diff + ' ' + me.unit;
							}
							
							// load the last called scenes from the property tree
							Ext.Ajax.request({
								url : '/json/property/getString?path=/scripts/metering/lastCalledScenes',
								// set the timeout to the refresh interval since we will do another request every 2 seconds
								timeout : DSS.addon.Metering.refreshInterval,
								success : function(response) {
									var jsonData = Ext.JSON.decode(response.responseText);
									if(jsonData.ok) {
										var jsonString = jsonData.result.value;
										var lastCalledScenes = Ext.JSON.decode(jsonString);

										//set an interval of 40 seconds to compare the timestamp of the last called scene with
										var now = new Date();
										var before = new Date(now);
										before.setTime(before.getTime() - 20000);
										var after = new Date(now);
										after.setTime(after.getTime() + 20000);
										
										var sceneNameFound = false;
										var sceneName;
										
										var zoneID;
										var groupID;
										var sceneID;
										
										for( i = lastCalledScenes.length-1; i >= 0; i--) {
											var time = new Date(lastCalledScenes[i].time);
											if(before <= time && time <= after) {
												zoneID = parseInt(lastCalledScenes[i].zoneID);
												groupID = parseInt(lastCalledScenes[i].groupID);
												sceneID = parseInt(lastCalledScenes[i].sceneID);
																																			
												sceneName = me.getSceneName(sceneID, groupID, zoneID);
												sceneNameFound = true;
												break;
											}
										}
																				
										if(sceneNameFound) {
											var sceneString;
											if (zoneID !== 0) {
												var zoneName = me.sceneNameStore.getAt(0).zones().getById(zoneID).get('text');
												sceneString = sceneName + ' ' + _("in Zone") + ' ' + zoneName;
											} else {
												sceneString = sceneName;
											}
											me.lineChart.series.get(0).addJumpMarker(me.meteringLineStore.getAt(me.meteringLineStore.getCount()-2), diffString, sceneString);
										} else {
											me.lineChart.series.get(0).addJumpMarker(me.meteringLineStore.getAt(me.meteringLineStore.getCount()-2), diffString);
										}
									}
								}
							});
						}
					}
				}
			});
		}, DSS.addon.Metering.refreshInterval);
	},
	
	stopThread : function() {
		clearInterval(this.thread);
		this.lineChart.series.get(0).removeAllMarkers();
	},
	
	getSceneName: function(sceneID, groupID, zoneID) {
		var me = this;		
		var sceneName;
		
		//if the zone of the last scene call is 0 we don't have to check if the zone is handled by the dsm
		if(zoneID === 0) {
			sceneName = DSS.addon.Metering.sceneNames[sceneID];
		} else if(groupID === 0) {
			sceneName = DSS.addon.Metering.sceneNames[sceneID];
		} else {
			//check if the zone of the matching scene call is handled by the selected dSM
			var selectedDSID = me.findParentByType('mainpanel').dSMView.getSelectedDSID();

			for( j = 0; j < me.apartmentStore.getAt(0).zones().getById(zoneID).devices().getCount(); j++) {
				var meterDSID = me.apartmentStore.getAt(0).zones().getById(zoneID).devices().getAt(0).get('meterDSID');
				if(meterDSID === selectedDSID) {
					if (me.sceneNameStore.getAt(0).zones().getById(zoneID).groups().getById(groupID).scenes().getById(sceneID)) {
						sceneName = me.sceneNameStore.getAt(0).zones().getById(zoneID).groups().getById(groupID).scenes().getById(sceneID).get('name');
					} else {
						sceneName = DSS.addon.Metering.sceneNames[sceneID];
					}
					break;
				}
			}
		}
		
		return sceneName;
	}


});


/*######################################
 MeteringLineSeries
 ######################################

Behaves like a normal LineSeries, but adds some functions to draw markers
to specific storeItems.
*/

Ext.define('DSS.addon.Metering.LineSeries', {

	type : 'Line',
	xField : 'date',
	yField : 'value',
	axis : ['left', 'bottom'],
	smooth : false,
	showMarkers : false,
	highlight : false,
	style : {
		fill : '#00523b',
		stroke : '#00523b',
		'stroke-width' : 3
	},
	

	constructor : function(config) {
		this.callParent(arguments);
		var me = this;
		Ext.apply(me, config, {
			tips : {
				trackMouse : true,
				width : 140,
				height : 35,
				renderer : function(storeItem, item) {
					this.setTitle(storeItem.get('value') + me.centerPanel.getUnit() + ' ' + _('at') + ' ' + Ext.Date.format(new Date(storeItem.get('date')), 'd.m.Y - G:i:s'));
				}
			}
		});
	},

	// array to store all markers
	jumpMarkers : [],
	
	// necessary reference to the centerPanel, to get the unit to display in the tooltips
	centerPanel : null,
	
	// add a new marker to the given store item. The text is displayed in the chart
	addJumpMarker : function(storeItem, diff, sceneName, color) {
		var me = this;

		for(var i = 0, ln = me.items.length; i < ln; i++) {
			if(storeItem.get('date') == me.items[i].value[0] && storeItem.get('value') == me.items[i].value[1]) {
				var diffSprite = Ext.create('Ext.draw.Sprite', {
					type : 'text',
					myType : 'diff',
					surface : me.chart.surface,
					text : diff,
					x : me.items[i].point[0] - 195,
					y : me.items[i].point[1] - 46,
					zIndex : 2,
					storeItem : storeItem
				});

				var frameSprite = Ext.create('Ext.draw.Sprite', {
					type : 'path',
					myType : 'frame',
					surface : me.chart.surface,
					path : "M 0 10 L 0 40 Q 0 50 10 50 L 170 50 L 200 60 L 180 40 L 180 10 Q 180 0 170 0 L 10 0 Q 0 0 0 10 Z",
					zIndex : 1,
					stroke : '#00523b',
					'stroke-width' : 3,
					fill : '#baefd1',
					opacity : 0.5,
					storeItem : storeItem
				});

				frameSprite.setAttributes({
					translate : {
						x : me.items[i].point[0] - 200,
						y : me.items[i].point[1] - 60
					}
				}, true);

				frameSprite.show(true);
				diffSprite.show(true);
				
				//check if a matching scene call was found
				if(sceneName) {
					var sceneSprite = Ext.create('Ext.draw.Sprite', {
						type : 'text',
						myType : 'scene',
						surface : me.chart.surface,
						text : sceneName,
						x : me.items[i].point[0] - 195,
						y : me.items[i].point[1] - 28,
						zIndex : 2,
						storeItem : storeItem
					});
					sceneSprite.show(true);
					me.jumpMarkers.push(sceneSprite);
				}
				
				me.jumpMarkers.push(frameSprite);
				me.jumpMarkers.push(diffSprite);
			}
		}
	}, 
	
	getPositionForStoreItem: function(storeItem) {
		var me = this;

		for(var i = 0, ln = me.items.length; i < ln; i++) {
			if(storeItem.get('date') == me.items[i].value[0] && storeItem.get('value') == me.items[i].value[1]) {
				return me.items[i].point
			}
		}
	},
	
	// sets all markers to the new position of the item. Necessary if a new item is added to the chart
	updateMarkers: function() {
		var me = this;

		for(var i = 0; i < me.jumpMarkers.length; i++) {
			var sprite = me.jumpMarkers[i];
			var point = me.getPositionForStoreItem(sprite.storeItem);
			if(point === undefined) {
				me.jumpMarkers.splice(i, 1);
				sprite.remove();
			} else {
				if(sprite.myType === 'diff') {
					sprite.setAttributes({
						x : point[0] - 195,
						y : point[1] - 46
					}, true);
				} else if (sprite.myType === 'scene') {
					sprite.setAttributes({
						x : point[0] - 195,
						y : point[1] - 28
					}, true);
				} else {
					sprite.setAttributes({
						translate : {
							x : point[0] - 200,
							y : point[1] - 60
						}
					}, true);
				}
			}
		}
	},
	
	removeAllMarkers: function() {
		var me = this;
		for(var i = 0; i < me.jumpMarkers.length; i++) {
			me.jumpMarkers[i].remove();
		}
	},
	
	setCenterPanel: function(centerPanel) {
		var me = this;
		me.centerPanel = centerPanel;
	}
});



/*######################################
 MeteringModel
######################################*/

Ext.define('DSS.addon.Metering.MeteringModel', {
	extend : 'Ext.data.Model',
	fields : [
		{name : 'date', type: 'date', dateFormat: 'timestamp', mapping: 0},
		{name : 'value', mapping: 1}
	]
});

/*######################################
 MeteringStore
######################################*/


Ext.define('DSS.addon.Metering.MeteringStore', {
	extend : 'Ext.data.Store',

	model : 'DSS.addon.Metering.MeteringModel',
	proxy : {
		type : 'ajax',
		actionMethods : {
			read : 'GET'
		},
		pageParam : undefined,
		limitParam : undefined,
		startParam : undefined,
		reader : {
			type : 'array',
			root : 'result.values',
			successProperty : 'ok'
		}
	},
	sorters : [{
		property : 'date',
		direction : 'ASC'
	}],
	// override remove method of this store so that no 'datachanged' or 'remove' event is fired
	// -> chart is only redrawed when a new value pair is added to the store
	// -> better performance
	remove : function(records, /* private */isMove) {
		if(!Ext.isArray(records)) {
			records = [records];
		}
		/*
		 * Pass the isMove parameter if we know we're going to be re-inserting this record
		 */
		isMove = isMove === true;
		var me = this, sync = false, i = 0, length = records.length, isPhantom, index, record;

		for(; i < length; i++) {
			record = records[i];
			index = me.data.indexOf(record);

			if(me.snapshot) {
				me.snapshot.remove(record);
			}
			if(index > -1) {
				isPhantom = record.phantom === true;
				if(!isMove && !isPhantom) {
					// don't push phantom records onto removed
					me.removed.push(record);
				}
				record.unjoin(me);
				me.data.remove(record);
				sync = sync || !isPhantom;
			}
		}

		if(!isMove && me.autoSync && sync) {
			me.sync();
		}
	}
});


/*######################################
 MeteringTheme
######################################*/

Ext.define('Ext.chart.theme.MeteringTheme', {
	extend : 'Ext.chart.theme.Base',

	constructor : function(config) {
		var baseColor = '#005138';
		//light green (like the borders)
		//var background = '#99e8bc';
		//darker green (like dSM Overview)
		//var fill = '#048c40';
		//very dark green (like the dS Logo)
		var fill = '#00523b';

		this.callParent([Ext.apply({
			axis : {
				fill : baseColor,
				stroke : baseColor
			},
			axisLabelLeft : {
				fill : baseColor
			},
			axisLabelBottom : {
				fill : baseColor
			},
			axisTitleLeft : {
				fill : baseColor
			},
			axisTitleBottom : {
				fill : baseColor
			},
			colors : [fill],
			markerThemes : [{
				fill : baseColor,
				type : 'circle'
			}]
		}, config)]);
	}
});


/*######################################
 MeteringMask
 
 Patch to fix the problem with missaligned zoom area
 see http://www.sencha.com/forum/showthread.php?155370-4.0.7-Chart-mask-%28for-selection-zooming%29-malfunctioning&highlight=chart+selection
######################################*/

Ext.override(Ext.chart.Mask, {
	require : ['Ext.chart.MaskLayer'],
	/**
	 * Creates new Mask.
	 * @param {Object} config (optional) Config object.
	 */
	constructor : function(config) {
		var me = this;

		me.addEvents('select');

		if(config) {
			Ext.apply(me, config);
		}
		if(me.mask) {
			me.on('afterrender', function() {
				//create a mask layer component
				var comp = Ext.create('Ext.chart.MaskLayer', {
					renderTo : me.el,
					// BEGIN SETH CHANGE
					maskType : me.mask
					// END SETH CHANGE
				});
				comp.el.on({
					// BEGIN SETH CHANGE
					'mousedown' : function(e) {
						// hack to hide tooltips
						// code duplicated below, but can't move to function because override
						// won't recognize it
						Ext.each(me.series.items, function(ser, idx) {
							if(ser.tooltip) {
								ser.savedTooltip = ser.tooltip;
								ser.tooltip = null;
							}
						});
					},
					// END SETH CHANGE
					'mousemove' : function(e) {
						me.onMouseMove(e);
						// BEGIN SETH CHANGE
						// used below in mouseup listener
						me.mouseMoved = true;
						// END SETH CHANGE
					},
					'mouseup' : function(e) {
						// BEGIN SETH CHANGE
						// the resizer may accidentally catch the first mouse up after mouse down
						// if the cursor happens to fall *within* the mask
						if(me.mouseDown)
							me.onMouseUp(e);
						else if(me.mouseMoved)
							// "resized" should be renamed "reselect"
							me.resized(e);
						// END SETH CHANGE
					}
				});
				// BEGIN SETH CHANGE
				var handles = 'all';
				switch (me.mask) {
					case 'horizontal':
						handles = 'e w';
						break;
					case 'vertical':
						handles = 'n s';
						break;
				}
				// END SETH CHANGE
				//create a resize handler for the component
				var resizeHandler = Ext.create('Ext.resizer.Resizer', {
					el : comp.el,
					// BEGIN SETH CHANGE
					handles : handles,
					pinned : false
					// END SETH CHANGE
				});
				resizeHandler.on({
					'resize' : function(e) {
						me.resized(e);
					}
				});
				// BEGIN SETH CHANGE
				// vertical or horizontal should not be able to be dragged around
				if(me.mask === true)
					comp.initDraggable();
				// END SETH CHANGE
				me.maskType = me.mask;
				me.mask = comp;
				me.maskSprite = me.surface.add({
					type : 'path',
					path : ['M', 0, 0],
					zIndex : 1001,
					opacity : 0.7,
					hidden : true,
					stroke : '#444'
				});
			}, me, {
				single : true
			});
		}
	},
	resized : function(e) {
		var me = this, bbox = me.bbox || me.chartBBox, x = bbox.x, y = bbox.y, width = bbox.width, height = bbox.height, box = me.mask.getBox(true), max = Math.max, min = Math.min, staticX = box.x - x, staticY = box.y - y;
		staticX = max(staticX, x);
		staticY = max(staticY, y);
		staticX = min(staticX, width);
		staticY = min(staticY, height);
		box.x = staticX;
		box.y = staticY;
		me.fireEvent('select', me, box);
	},
	onMouseUp : function(e) {
		var me = this, bbox = me.bbox || me.chartBBox, sel = me.maskSelection;
		me.maskMouseDown = false;
		me.mouseDown = false;
		if(me.mouseMoved) {
			me.onMouseMove(e);
			me.mouseMoved = false;
			me.fireEvent('select', me, {
				x : sel.x - bbox.x,
				y : sel.y - bbox.y,
				width : sel.width,
				height : sel.height
			});
		}

		// BEGIN SETH CHANGE
		// hack to undo hack to hide tooltips
		Ext.each(me.series.items, function(ser, idx) {
			if(ser.savedTooltip) {
				ser.tooltip = ser.savedTooltip;
			}
		});
		// END SETH CHANGE
	},
	onMouseDown : function(e) {
		var me = this;
		me.mouseDown = true;
		me.mouseMoved = false;
		me.maskMouseDown = {
			x : e.getPageX() - me.el.getX(),
			y : e.getPageY() - me.el.getY()
		};

		// BEGIN SETH CHANGE
		// hack to hide tooltips
		Ext.each(me.series.items, function(ser, idx) {
			if(ser.tooltip) {
				ser.savedTooltip = ser.tooltip;
				ser.tooltip = null;
			}
		});
		// END SETH CHANGE
	},
	onMouseMove : function(e) {
		var me = this, mask = me.maskType, bbox = me.bbox || me.chartBBox, x = bbox.x, y = bbox.y, math = Math, floor = math.floor, abs = math.abs, min = math.min, max = math.max, height = floor(y + bbox.height), width = floor(x + bbox.width), posX = e.getPageX(), posY = e.getPageY(), staticX = posX - me.el.getX(), staticY = posY - me.el.getY(), maskMouseDown = me.maskMouseDown, path;

		me.mouseMoved = me.mouseDown;
		staticX = max(staticX, x);
		staticY = max(staticY, y);
		staticX = min(staticX, width);
		staticY = min(staticY, height);
		if(maskMouseDown && me.mouseDown) {
			if(mask == 'horizontal') {
				staticY = y;
				maskMouseDown.y = height;
				// BEGIN SETH CHANGE
				posY = me.el.getY() + me.insetPadding;
				// END SETH CHANGE
			} else if(mask == 'vertical') {
				staticX = x;
				maskMouseDown.x = width;
				// BEGIN SETH CHANGE
				posX = me.el.getX() + me.insetPadding;
				// END SETH CHANGE
			}
			width = maskMouseDown.x - staticX;
			height = maskMouseDown.y - staticY;
			path = ['M', staticX, staticY, 'l', width, 0, 0, height, -width, 0, 'z'];
			// this x and y are relative to the chart
			me.maskSelection = {
				x : width > 0 ? staticX : staticX + width,
				y : height > 0 ? staticY : staticY + height,
				width : abs(width),
				height : abs(height)
			};

			// BEGIN SETH CHANGE
			// this x and y must be relative to the page
			me.mask.updateBox({
				// if dragging left-to-right, go from the click to the cursor
				// if dragging right-to-left, go from the cursor to the click
				x : posX + (width < 0 ? width : 0),
				y : posY + (height < 0 ? height : 0),
				width : abs(width),
				height : abs(height)
			});
			// END SETH CHANGE

			me.mask.show();
			me.maskSprite.setAttributes({
				hidden : true
			}, true);
		} else {
			if(mask == 'horizontal') {
				path = ['M', staticX, y, 'L', staticX, height];
			} else if(mask == 'vertical') {
				path = ['M', x, staticY, 'L', width, staticY];
			} else {
				path = ['M', staticX, y, 'L', staticX, height, 'M', x, staticY, 'L', width, staticY];
			}
			me.maskSprite.setAttributes({
				path : path,
				fill : me.maskMouseDown ? me.maskSprite.stroke : false,
				'stroke-width' : mask === true ? 1 : 3,
				hidden : false
			}, true);
		}
	},
	onMouseLeave : function(e) {
		var me = this;
		me.mouseMoved = false;
		me.mouseDown = false;
		me.maskMouseDown = false;
		// me.mask.hide();
		// me.maskSprite.hide(true);
	}
});

// overriding to control styles
Ext.override(Ext.chart.MaskLayer, {
	extend : 'Ext.Component',

	constructor : function(config) {
		// BEGIN SETH CHANGE
		// only show a move cursor if you should be moving the mask
		var style = 'position:absolute;background-color:#888;opacity:0.4;border:1px solid #222;';
		if(config.maskType === true)
			style += 'cursor:move;';
		// END SETH CHANGE
		config = Ext.apply(config || {}, {
			style : style
		});
		this.callParent([config]);
	},
	initComponent : function() {
		var me = this;
		me.callParent(arguments);
		me.addEvents('mousedown', 'mouseup', 'mousemove', 'mouseenter', 'mouseleave');
	},
	initDraggable : function() {
		this.callParent(arguments);
		this.dd.onStart = function(e) {
			var me = this, comp = me.comp;

			// Cache the start [X, Y] array
			this.startPosition = comp.getPosition(true);

			// If client Component has a ghost method to show a lightweight version of itself
			// then use that as a drag proxy unless configured to liveDrag.
			if(comp.ghost && !comp.liveDrag) {
				me.proxy = comp.ghost();
				me.dragTarget = me.proxy.header.el;
			}

			// Set the constrainTo Region before we start dragging.
			if(me.constrain || me.constrainDelegate) {
				me.constrainTo = me.calculateConstrainRegion();
			}
		};
	}
});
